@@ -53,7 +53,7 @@ enum {
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_EXCHANGE
 static ECalBackendExchange *parent_class = NULL;
 
-#define d(x) 
+#define d(x)
 
 static ECalBackendSyncStatus modify_object_with_href (ECalBackendSync *backend, EDataCal *cal, const char *calobj, CalObjModType mod, char **old_object, char **new_object, const char *href, const char *rid_to_remove);
 
@@ -134,7 +134,7 @@ add_vevent (ECalBackendExchange *cbex,
 			 */
 			if (transp) {
 				icalcomponent_remove_property (icalcomp,transp);
-				icalproperty_free (transp); 
+				icalproperty_free (transp);
 				transp = NULL;
 			}
 
@@ -178,13 +178,13 @@ add_vevent (ECalBackendExchange *cbex,
 	/* Now add to the cache */
 	status = e_cal_backend_exchange_add_object (cbex, href, lastmod, icalcomp);
 	e_cal_backend_exchange_cache_unlock (cbex);
-	
+
 	return status;
 }
 
 /* Add the event to the cache, Notify the backend if it is sucessfully added */
 static gboolean
-add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod, 
+add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 	  const char *uid, const char *body, int len, int receipts)
 {
 	const char *start, *end;
@@ -268,7 +268,7 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 
 		if (new_comp) {
 			status = add_vevent (cbex, href, lastmod, new_comp);
-			
+
 			if (status) {
 				char *object = g_strdup (icalcomponent_as_ical_string (new_comp));
 				e_cal_backend_notify_object_created (backend, object);
@@ -294,14 +294,14 @@ static const char *event_properties[] = {
 	E2K_PR_DAV_LAST_MODIFIED,
 	E2K_PR_HTTPMAIL_HAS_ATTACHMENT,
 	PR_READ_RECEIPT_REQUESTED,
-	PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED	
+	PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED
 };
 static const int n_event_properties = G_N_ELEMENTS (event_properties);
 
 static const char *new_event_properties[] = {
 	PR_INTERNET_CONTENT,
 	PR_READ_RECEIPT_REQUESTED,
-	PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED	
+	PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED
 };
 static const int n_new_event_properties = G_N_ELEMENTS (new_event_properties);
 
@@ -375,14 +375,14 @@ get_changed_events (ECalBackendExchange *cbex)
 		modtime = e2k_properties_get_prop (result->props,
 						   E2K_PR_DAV_LAST_MODIFIED);
 
-		attach_prop = e2k_properties_get_prop (result->props, 
+		attach_prop = e2k_properties_get_prop (result->props,
 						E2K_PR_HTTPMAIL_HAS_ATTACHMENT);
 
 		receipts = e2k_properties_get_prop (result->props,
 						PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED);
 		if (receipts && atoi (receipts))
 			status_tracking = EX_DELIVERED_RECEIPTS;
-	
+
 		receipts = NULL;
 		receipts = e2k_properties_get_prop (result->props,
 						PR_READ_RECEIPT_REQUESTED);
@@ -458,7 +458,7 @@ get_changed_events (ECalBackendExchange *cbex)
 					PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED);
 		if (receipts && atoi (receipts))
 			status_tracking = EX_DELIVERED_RECEIPTS;
-	
+
 		receipts = NULL;
 		receipts = e2k_properties_get_prop (result->props,
 					PR_READ_RECEIPT_REQUESTED);
@@ -536,12 +536,12 @@ notify_changes (E2kContext *ctx, const char *uri,
 {
 
 	ECalBackendExchange *ecalbex = E_CAL_BACKEND_EXCHANGE (user_data);
-	
+
 	g_return_if_fail (E_IS_CAL_BACKEND_EXCHANGE (ecalbex));
 	g_return_if_fail (uri != NULL);
 
 	get_changed_events (ecalbex);
-	
+
 }
 
 static ECalBackendSyncStatus
@@ -575,7 +575,7 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal,
 	if (!thread) {
 		g_warning (G_STRLOC ": %s", error->message);
 		g_error_free (error);
-		
+
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 	return GNOME_Evolution_Calendar_Success;
@@ -612,69 +612,6 @@ add_timezone_cb (icalparameter *param, void *data)
 		icalcomponent_add_component (cbdata->vcal_comp, vtzcomp);
 }
 
-static void
-process_delegated_cal_object (icalcomponent *icalcomp, char *delegator_name, char *delegator_email, char *delegatee_email)
-{
-	icalproperty *prop = NULL;
-
-	prop = icalcomponent_get_first_property (icalcomp, ICAL_ORGANIZER_PROPERTY);
-	if (prop) {
-		const char *organizer;
-		char *text = NULL;
-
-		organizer = icalproperty_get_value_as_string (prop);
-		if (organizer) {
-			if (!g_ascii_strncasecmp (organizer, "mailto:", 7))
-				text = g_strdup (organizer+7);
-
-			text = g_strstrip (text);
-			if (text && (!g_ascii_strcasecmp (delegatee_email, text) || !g_ascii_strcasecmp (delegator_email, text))) {
-				icalproperty_set_organizer (prop, g_strdup_printf ("MAILTO:%s", delegator_email));
-				icalproperty_remove_parameter_by_kind (prop, ICAL_CN_PARAMETER);
-				icalproperty_add_parameter (prop, icalparameter_new_cn (g_strdup(delegator_name)));
-				icalproperty_remove_parameter_by_kind (prop, ICAL_SENTBY_PARAMETER);
-				icalproperty_add_parameter (prop, icalparameter_new_sentby (g_strdup_printf("MAILTO:%s", delegatee_email)));
-				prop = find_attendee_prop (icalcomp, delegator_email);
-				if (prop != NULL) {
-					icalcomponent_remove_property (icalcomp, prop);
-					icalproperty_free (prop);
-				}
-				g_free (text);
-				return; 
-			}
-			g_free (text);
-		}
-	}
-	prop = NULL;
-
-	for (prop = icalcomponent_get_first_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
-	     prop != NULL;
-	     prop = icalcomponent_get_next_property (icalcomp, ICAL_ATTENDEE_PROPERTY)) {
-		const char *attendee;
-		char *text = NULL;
-
-		attendee = icalproperty_get_value_as_string (prop);
-		if (!attendee)
-			continue;
-
-		if (!g_ascii_strncasecmp (attendee, "mailto:", 7))
-			text = g_strdup (attendee+7);
-
-		text = g_strstrip (text);
-		if (text && !g_ascii_strcasecmp (delegator_email, text)) {
-			icalproperty_remove_parameter_by_kind (prop, ICAL_CN_PARAMETER);
-			icalproperty_add_parameter (prop, icalparameter_new_cn (g_strdup(delegator_name)));
-			icalproperty_remove_parameter_by_kind (prop, ICAL_SENTBY_PARAMETER);
-			icalproperty_add_parameter (prop, icalparameter_new_sentby (g_strdup_printf("MAILTO:%s", delegatee_email)));
-			g_free (text);
-			break;
-		}
-		g_free (text);
-	}
-
-	return;
-}
-
 gboolean
 check_for_send_options (icalcomponent *icalcomp, E2kProperties *props)
 {
@@ -682,7 +619,7 @@ check_for_send_options (icalcomponent *icalcomp, E2kProperties *props)
 	gboolean exists = FALSE;
 	const char *x_name, *x_val;
 
-	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);	
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
 	while (icalprop && !exists) {
 		x_name = icalproperty_get_x_name (icalprop);
 		if (!strcmp(x_name, "X-EVOLUTION-OPTIONS-TRACKINFO")) {
@@ -691,16 +628,16 @@ check_for_send_options (icalcomponent *icalcomp, E2kProperties *props)
 			switch (atoi (x_val)) {
 				case EX_ALL: /* Track if delivered and opened */
 				case EX_READ_AND_DELIVERED: /* Track if delivered and opened */
-					e2k_properties_set_int (props, 
+					e2k_properties_set_int (props,
 					PR_READ_RECEIPT_REQUESTED, 1);
 					/* Fall Through */
 				case EX_DELIVERED_RECEIPTS : /* Track if delivered */
-					e2k_properties_set_int (props, 
+					e2k_properties_set_int (props,
 					PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED, 1);
 					break;
 				default : /* None */
 					exists = FALSE;
-					break; 
+					break;
 			}
 		}
 		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
@@ -748,16 +685,16 @@ check_owner_partstatus_for_declined (ECalBackendSync *backend,
 	icalproperty *icalprop;
 	icalparameter *param;
 	gchar *email;
-	
+
 	email = e_cal_backend_exchange_get_owner_email (backend);
 	icalprop = find_attendee_prop (icalcomp, email);
 
 	g_free (email);
 	if (!icalprop)
 		return FALSE;
-	
+
 	param = icalproperty_get_first_parameter (icalprop, ICAL_PARTSTAT_PARAMETER);
-	
+
 	if (icalparameter_get_partstat (param) == ICAL_PARTSTAT_DECLINED) {
 		return TRUE;
 	}
@@ -790,12 +727,12 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	E2kContext *e2kctx;
 	struct _cb_data *cbdata;
 	gboolean send_options;
-	
+
 	d(printf ("ecbexc_create_object(%p, %p, %s, %s)", backend, cal, *calobj ? *calobj : NULL, *uid ? *uid : NULL));
 
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 	cbex = E_CAL_BACKEND_EXCHANGE (backend);
-	
+
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
 
@@ -807,19 +744,19 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	/* check for permission denied:: priv->writable??
 	   ....
 	 */
-	
+
 	icalcomp = icalparser_parse_string (*calobj);
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
-	
+
 	kind = icalcomponent_isa (icalcomp);
-	
+
 	if (kind != e_cal_backend_get_kind (E_CAL_BACKEND (backend))) {
-		
+
 		icalcomponent_free (icalcomp);
 		return GNOME_Evolution_Calendar_InvalidObject;
 	}
-	
+
 	temp_comp_uid = icalcomponent_get_uid (icalcomp);
 	if (!temp_comp_uid) {
 		icalcomponent_free (icalcomp);
@@ -832,7 +769,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 		return ;
 	}
 	#endif
-	
+
 	/* Delegated calendar */
 	if (g_ascii_strcasecmp(e_cal_backend_exchange_get_owner_email (backend), exchange_account_get_email_id (cbex->account)))
 		process_delegated_cal_object (icalcomp, e_cal_backend_exchange_get_owner_name (backend), e_cal_backend_exchange_get_owner_email (backend), exchange_account_get_email_id (cbex->account));
@@ -844,24 +781,24 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	current = icaltime_from_timet (time (NULL), 0);
 	icalcomponent_add_property (icalcomp, icalproperty_new_created (current));
 	icalcomponent_add_property (icalcomp, icalproperty_new_lastmodified (current));
-	
+
 	/* Fetch summary */
 	summary = icalcomponent_get_summary (icalcomp);
 	if (!summary)
 		summary = "";
-	
+
 	lastmod = e2k_timestamp_from_icaltime (current);
-	
+
 	comp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (comp, icalcomp);	
-	
+	e_cal_component_set_icalcomponent (comp, icalcomp);
+
 	/* Check for attachments */
 	if (e_cal_component_has_attachments (comp)) {
 		d(printf ("This comp has attachments !!\n"));
 		attach_body = build_msg (E_CAL_BACKEND_EXCHANGE (cbexc), comp, summary, &boundary);
-		attach_body_crlf = e_cal_backend_exchange_lf_to_crlf (attach_body);	
+		attach_body_crlf = e_cal_backend_exchange_lf_to_crlf (attach_body);
 	}
-	
+
 	update_x_properties (E_CAL_BACKEND_EXCHANGE (cbexc), comp);
 
 	cbdata = g_new0 (struct _cb_data, 1);
@@ -870,7 +807,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	cbdata->cal = cal;
 
 	/* Remove X parameters from properties */
-	/* This is specifically for X-EVOLUTION-END-DATE, 
+	/* This is specifically for X-EVOLUTION-END-DATE,
 	   but removing anything else is probably ok too */
 	for (icalprop = icalcomponent_get_first_property (icalcomp, ICAL_ANY_PROPERTY);
 	     icalprop != NULL;
@@ -878,9 +815,9 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	{
 		icalproperty_remove_parameter (icalprop, ICAL_X_PARAMETER);
 	}
-		
+
 	/* add the timezones information and the component itself
-	   to the VCALENDAR object */	
+	   to the VCALENDAR object */
 	e_cal_component_commit_sequence (comp);
 	*calobj = e_cal_component_get_as_string (comp);
 	if (!*calobj) {
@@ -892,15 +829,15 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	real_icalcomp = icalparser_parse_string (*calobj);
 
 	icalcomponent_foreach_tzid (real_icalcomp, add_timezone_cb, cbdata);
-	icalcomponent_add_component (cbdata->vcal_comp, real_icalcomp);	
-	
+	icalcomponent_add_component (cbdata->vcal_comp, real_icalcomp);
+
 	body = icalcomponent_as_ical_string (cbdata->vcal_comp);
-	body_crlf = e_cal_backend_exchange_lf_to_crlf (body);	
-		
+	body_crlf = e_cal_backend_exchange_lf_to_crlf (body);
+
 	date = e_cal_backend_exchange_make_timestamp_rfc822 (time (NULL));
 	if (!g_ascii_strcasecmp(e_cal_backend_exchange_get_owner_email (backend), exchange_account_get_email_id (cbex->account)))
 		from = e_cal_backend_exchange_get_from_string (backend, comp);
-	else 
+	else
 		from = e_cal_backend_exchange_get_sender_string (backend, comp);
 
 	if (attach_body) {
@@ -941,9 +878,9 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 				       body_crlf);
 	}
 
-	http_status = e_folder_exchange_put_new (E_CAL_BACKEND_EXCHANGE (cbexc)->folder, NULL, summary, 
-						NULL, NULL, "message/rfc822", 
-						msg, strlen(msg), &location, &ru_header);	
+	http_status = e_folder_exchange_put_new (E_CAL_BACKEND_EXCHANGE (cbexc)->folder, NULL, summary,
+						NULL, NULL, "message/rfc822",
+						msg, strlen(msg), &location, &ru_header);
 
 	if ((http_status == E2K_HTTP_CREATED) && send_options) {
 		e2kctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);
@@ -953,7 +890,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	g_free (date);
 	g_free (from);
 	g_free (body_crlf);
-	g_free (msg); 
+	g_free (msg);
 	icalcomponent_free (cbdata->vcal_comp); // not sure
 	g_free (cbdata);
 
@@ -968,8 +905,8 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	/*add object*/
 	e_cal_backend_exchange_add_object (E_CAL_BACKEND_EXCHANGE (cbexc), location, lastmod, icalcomp);
 	e_cal_backend_exchange_cache_unlock (cbex);
-	*uid = g_strdup (temp_comp_uid);	
-		
+	*uid = g_strdup (temp_comp_uid);
+
 	g_object_unref (comp);
 	g_free (lastmod);
 	g_free (location);
@@ -1062,9 +999,9 @@ update_x_properties (ECalBackendExchange *cbex, ECalComponent *comp)
 		icalcomponent_remove_property (icalcomp, icalprop);
 		icalproperty_free (icalprop);
 	}
-	
+
 	g_slist_free (props);
-	
+
 	/* Now set the ones that weren't set. */
 	if (!(prop_set & BUSYSTATUS)) {
 		icalprop = icalproperty_new_x (busystatus);
@@ -1143,7 +1080,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
-	
+
 	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
                 /* FIXME */
                 return GNOME_Evolution_Calendar_InvalidObject;
@@ -1155,9 +1092,9 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	icalcomp = icalparser_parse_string (calobj);
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
-	
+
 	kind = icalcomponent_isa (icalcomp);
-	
+
 	if (kind != e_cal_backend_get_kind (E_CAL_BACKEND (backend))) {
 		icalcomponent_free (icalcomp);
 		return GNOME_Evolution_Calendar_InvalidObject;
@@ -1166,18 +1103,18 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 	if (!remove)
 		key_rid = icalcomponent_get_recurrenceid (icalcomp);
-	else  
+	else
 		key_rid = icaltime_from_string (rid_to_remove);
-	
+
 	e_cal_backend_exchange_cache_lock (cbex);
 	ecomp = get_exchange_comp (cbex, comp_uid);
-	
+
 	if (!ecomp){
 		icalcomponent_free (icalcomp);
 		e_cal_backend_exchange_cache_unlock (cbex);
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	}
-	
+
 	/* Fetch summary */
 	summary = icalcomponent_get_summary (icalcomp);
 	if (!summary)
@@ -1192,7 +1129,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		if (!key_rid.zone)
 			key_rid.zone = start.zone;
 
-		/* This means its a instance generated from master object. So replace 
+		/* This means its a instance generated from master object. So replace
 		   the dates stored dates from the master object */
 
 		if (icaltime_compare_date_only (start, key_rid) == 0) {
@@ -1202,12 +1139,12 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 			m_start = icalcomponent_get_dtstart (ecomp->icomp);
 			m_end = icalcomponent_get_dtend (ecomp->icomp);
 			end = icalcomponent_get_dtend (icalcomp);
-	
+
 			if (icaltime_compare (start, key_rid) != 0) {
 					m_start.hour = start.hour;
 					m_start.minute = start.minute;
 					m_start.second = start.second;
-						
+
 			}
 
 			if (!((m_end.hour == end.hour) && (m_end.minute == end.minute) && (m_end.second == end.second))) {
@@ -1218,7 +1155,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 			icalcomponent_set_dtstart (icalcomp, m_start);
 			icalcomponent_set_dtend (icalcomp, m_end);
-			
+
 			prop = icalcomponent_get_first_property (icalcomp, ICAL_RECURRENCEID_PROPERTY);
 			icalcomponent_remove_property (icalcomp, prop);
 			icalproperty_free (prop);
@@ -1229,14 +1166,14 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	e_cal_component_set_icalcomponent (updated_ecomp, icalcomp);
 
 	update_x_properties (E_CAL_BACKEND_EXCHANGE (cbexc), updated_ecomp);
-	
+
 	last_modified = icaltime_from_timet (time (NULL), 0);
 	e_cal_component_set_last_modified (updated_ecomp, &last_modified);
 
 	if (e_cal_component_has_attachments (updated_ecomp)) {
 		d(printf ("This comp has attachments !!\n"));
 		attach_body = build_msg (E_CAL_BACKEND_EXCHANGE (cbexc), updated_ecomp, summary, &boundary);
-		attach_body_crlf = e_cal_backend_exchange_lf_to_crlf (attach_body);	
+		attach_body_crlf = e_cal_backend_exchange_lf_to_crlf (attach_body);
 	}
 
 	e_cal_component_commit_sequence (updated_ecomp);
@@ -1257,7 +1194,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	send_options = check_for_send_options (updated_icalcomp, props);
 
 	/* Remove X parameters from properties */
-	/* This is specifically for X-EVOLUTION-END-DATE, 
+	/* This is specifically for X-EVOLUTION-END-DATE,
 	   but removing anything else is probably ok too */
 	for (icalprop = icalcomponent_get_first_property (updated_icalcomp, ICAL_ANY_PROPERTY);
 	     icalprop != NULL;
@@ -1277,7 +1214,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	cbdata = g_new0 (struct _cb_data, 1);
 	cbdata->be = backend;
 	cbdata->vcal_comp = e_cal_util_new_top_level ();
-	cbdata->cal = cal;	
+	cbdata->cal = cal;
 
 	e_cal_component_get_dtstart (real_ecomp, &dt);
 	if (dt.value->is_date) {
@@ -1291,7 +1228,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		dt.value->is_utc = FALSE;
 		dt.value->hour = dt.value->minute = dt.value->second = 0;
 		dt.value->zone = zone;
-		
+
 		g_free ((char *)dt.tzid);
 		dt.tzid = g_strdup (icaltimezone_get_tzid (zone));
 		e_cal_component_set_dtstart (real_ecomp, &dt);
@@ -1314,13 +1251,13 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	    && e_cal_component_has_simple_recurrence (real_ecomp)) {
 		GSList *rrule_list;
 		struct icalrecurrencetype *r;
-		
+
 		e_cal_component_get_rrule_list (real_ecomp, &rrule_list);
 		r = rrule_list->data;
 
 		if (!icaltime_is_null_time (r->until) && r->until.is_date) {
 			icaltimezone *from_zone, *to_zone;
-			
+
 			e_cal_component_get_dtstart (real_ecomp, &dt);
 
 			if (dt.tzid == NULL)
@@ -1337,11 +1274,11 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 			r->until.minute = dt.value->minute;
 			r->until.second = dt.value->second;
 			r->until.is_date = FALSE;
-			
+
 			icaltimezone_convert_time (&r->until, from_zone, to_zone);
 			r->until.is_utc = TRUE;
-			
-			e_cal_component_set_rrule_list (real_ecomp, rrule_list);			
+
+			e_cal_component_set_rrule_list (real_ecomp, rrule_list);
 			e_cal_component_free_datetime (&dt);
 		}
 
@@ -1361,7 +1298,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	real_comp_str = e_cal_component_get_as_string (real_ecomp);
 	if (!real_comp_str) {
 		g_object_unref (real_ecomp);
-		g_object_unref (updated_ecomp);	
+		g_object_unref (updated_ecomp);
 		icalcomponent_free (cbdata->vcal_comp);
 		g_free (cbdata);
 		e_cal_backend_exchange_cache_unlock (cbex);
@@ -1375,14 +1312,14 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	/* Master object should be added first */
 	if (!remove && mod == CALOBJ_MOD_ALL)
 		icalcomponent_add_component (cbdata->vcal_comp, real_icalcomp);
-	
+
 
 	/* We need to add all the instances to the VCalendar component while sending to
 	   the server, so that we don't lose any detached instances */
 	if (ecomp->icomp && mod == CALOBJ_MOD_THIS) {
 		icalcomponent_add_component (cbdata->vcal_comp, icalcomponent_new_clone (ecomp->icomp));
 	}
-	
+
 	for (l = ecomp->instances; l != NULL; l = l->next) {
 		icalcomponent *icomp = l->data;
 
@@ -1392,7 +1329,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 			e_cal_component_set_icalcomponent (cached_ecomp,
 					icalcomponent_new_clone (icomp));
-			if (remove) 
+			if (remove)
 				to_remove = l;
 			continue;
 		} else {
@@ -1401,7 +1338,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 			update_x_properties (E_CAL_BACKEND_EXCHANGE (cbexc), comp);
 
-			icalcomponent_add_component (cbdata->vcal_comp, 
+			icalcomponent_add_component (cbdata->vcal_comp,
 					icalcomponent_new_clone (e_cal_component_get_icalcomponent (comp)));
 
 			g_object_unref (comp);
@@ -1423,13 +1360,13 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 	body = icalcomponent_as_ical_string (cbdata->vcal_comp);
 	body_crlf = e_cal_backend_exchange_lf_to_crlf (body);
-	
+
 	date = e_cal_backend_exchange_make_timestamp_rfc822 (time (NULL));
 	if (!g_ascii_strcasecmp(e_cal_backend_exchange_get_owner_email (backend), exchange_account_get_email_id (cbex->account)))
 		from = e_cal_backend_exchange_get_from_string (backend, updated_ecomp);
-	else 
+	else
 		from = e_cal_backend_exchange_get_sender_string (backend, updated_ecomp);
-	
+
 	if (attach_body) {
 		msg = g_strdup_printf ("Subject: %s\r\n"
 				       "Date: %s\r\n"
@@ -1468,7 +1405,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 				       from ? from : "Evolution",
 				       body_crlf);
 	}
-	
+
 	g_free (date);
 	g_free (from);
 	g_free (body_crlf);
@@ -1478,8 +1415,8 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		*old_object = e_cal_component_get_as_string (cached_ecomp);
 	}
 
-	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
-	
+	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);
+
 	/* PUT the iCal object in the Exchange server */
 	if (href)
 		new_href = href;
@@ -1494,19 +1431,19 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (http_status)) {
 		e_cal_backend_exchange_cache_lock (cbex);
-		e_cal_backend_exchange_modify_object (E_CAL_BACKEND_EXCHANGE (cbexc), 
+		e_cal_backend_exchange_modify_object (E_CAL_BACKEND_EXCHANGE (cbexc),
 							e_cal_component_get_icalcomponent (real_ecomp), mod, remove);
 		e_cal_backend_exchange_cache_unlock (cbex);
-	
+
 		if (!remove)
 			*new_object = e_cal_component_get_as_string (real_ecomp);
-	} else 
-	 	status = GNOME_Evolution_Calendar_OtherError;		
+	} else
+	 	status = GNOME_Evolution_Calendar_OtherError;
 
 	g_free (msg);
 	g_object_unref (real_ecomp);
 	g_object_unref (updated_ecomp);
-	
+
 	if (cached_ecomp)
 		g_object_unref (cached_ecomp);
 
@@ -1516,7 +1453,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 	return status;
 }
- 
+
 static ECalBackendSyncStatus
 remove_object (ECalBackendSync *backend, EDataCal *cal,
 	       const char *uid, const char *rid, CalObjModType mod,
@@ -1531,11 +1468,11 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 	char *calobj, *obj = NULL;
 	struct icaltimetype time_rid;
 	ECalBackendSyncStatus ebs_status;
-	
+
 	/* Will handle only CALOBJ_MOD_THIS and CALOBJ_MOD_ALL for mod.
 	   CALOBJ_MOD_THISANDPRIOR and CALOBJ_MOD_THISANDFUTURE are not
 	   handled. */
-	
+
 	cbexc = E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 	cbex = E_CAL_BACKEND_EXCHANGE (backend);
 
@@ -1548,12 +1485,12 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 
 	e_cal_backend_exchange_cache_lock (cbex);
 	ecomp = get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbexc), uid);
-	
+
 	if (!ecomp) {
 		e_cal_backend_exchange_cache_unlock (cbex);
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	}
-		
+
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->icomp));
 	if (old_object) {
@@ -1561,15 +1498,15 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 		*old_object = e_cal_component_get_as_string (comp);
 	}
 
-	/*TODO How handle multiple detached intances with no master object ?*/	
+	/*TODO How handle multiple detached intances with no master object ?*/
 	if (mod == CALOBJ_MOD_THIS && rid && *rid && ecomp->icomp) {
 		char *new_object = NULL;
-		
+
 		/*remove a single instance of a recurring event and modify */
 		time_rid = icaltime_from_string (rid);
 		e_cal_util_remove_instances (ecomp->icomp, time_rid, mod);
 		calobj  = (char *) icalcomponent_as_ical_string (ecomp->icomp);
-		
+
 		e_cal_backend_exchange_cache_unlock (cbex);
 		ebs_status = modify_object_with_href (backend, cal, calobj, mod, &obj, &new_object, NULL, rid);
 		g_object_unref (comp);
@@ -1579,17 +1516,17 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 			g_free (*old_object);
 			*old_object = obj;
 		}
-		
+
 		g_free (new_object);
 
 		return ebs_status;
-	} else 
+	} else
 		e_cal_backend_exchange_cache_unlock (cbex);
 
 	g_object_unref (comp);
 
-	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
-	
+	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);
+
 	status = e2k_context_delete (ctx, NULL, ecomp->href);
 	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		e_cal_backend_exchange_cache_lock (cbex);
@@ -1617,7 +1554,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 	struct icaltimetype current;
 	icalproperty_method method;
 	icalcomponent *subcomp, *icalcomp;
-	ECalBackendSyncStatus status = GNOME_Evolution_Calendar_Success;	
+	ECalBackendSyncStatus status = GNOME_Evolution_Calendar_Success;
 
 	d(printf ("ecbexc_modify_object(%p, %p, %s)", backend, cal, calobj ? calobj : NULL));
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
@@ -1625,7 +1562,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
-	
+
 	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
                 /* FIXME */
                 return GNOME_Evolution_Calendar_InvalidObject;
@@ -1635,7 +1572,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 	status = e_cal_backend_exchange_extract_components (calobj, &method, &comps);
 	if (status != GNOME_Evolution_Calendar_Success)
 		return GNOME_Evolution_Calendar_InvalidObject;
-		
+
 	icalcomp = icalparser_parse_string (calobj);
 	add_timezones_from_comp (E_CAL_BACKEND_EXCHANGE (backend), icalcomp);
 	icalcomponent_free (icalcomp);
@@ -1644,17 +1581,17 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		const char *uid, *rid;
 		char *icalobj;
 		char *object = NULL;
-		
+
 		subcomp = l->data;
-		
+
 		comp = e_cal_component_new ();
 		e_cal_component_set_icalcomponent (comp, subcomp);
-		
+
 		/*create time and last modified*/
 		current = icaltime_from_timet (time (NULL), 0);
 		e_cal_component_set_created (comp, &current);
 		e_cal_component_set_last_modified (comp, &current);
-		
+
 		e_cal_component_get_uid (comp, &uid);
 		rid = e_cal_component_get_recurid_as_string (comp);
 
@@ -1671,7 +1608,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 				e_cal_backend_exchange_cache_unlock (cbex);
 				if (check_owner_partstatus_for_declined (backend, subcomp)) {
 					ECalComponentId *id = NULL;
-					status = remove_object (backend, cal, uid, NULL, 
+					status = remove_object (backend, cal, uid, NULL,
 								CALOBJ_MOD_ALL, &old_object,
 								NULL);
 					if (status != GNOME_Evolution_Calendar_Success)
@@ -1683,7 +1620,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 				} else {
 					char *new_object = NULL;
 					CalObjModType mod = CALOBJ_MOD_ALL;
-					
+
 					if (e_cal_util_component_is_instance (subcomp))
 						mod = CALOBJ_MOD_THIS;
 
@@ -1707,7 +1644,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 				d(printf ("object : %s .. not found in the cache\n", uid));
 				icalobj = (char *) icalcomponent_as_ical_string (subcomp);
 				d(printf ("Create a new object : %s\n", icalobj));
-				
+
 				e_cal_backend_exchange_cache_unlock (cbex);
 				status = create_object (backend, cal, &icalobj, &returned_uid);
 				if (status != GNOME_Evolution_Calendar_Success)
@@ -1737,7 +1674,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 				ECalComponentId *id = e_cal_component_get_id (comp);
 				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), id, icalobj, NULL);
 				e_cal_component_free_id (id);
-				
+
 			}
 			if (object) {
 				g_free (object);
@@ -1752,8 +1689,8 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 	}
 	g_list_free (comps);
 	return status;
-		
- error:	
+
+ error:
 	if (comp)
 		g_object_unref (comp);
 	return status;
@@ -1767,7 +1704,7 @@ typedef enum {
 	E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR
 } ECalBackendExchangeBookingResult;
 
-												 
+
 /* start_time and end_time are in e2k_timestamp format. */
 static ECalBackendExchangeBookingResult
 book_resource (ECalBackendExchange *cbex,
@@ -1799,14 +1736,14 @@ book_resource (ECalBackendExchange *cbex,
 	int nresult;
 	ECalBackendExchangeBookingResult retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
 	const char *localfreebusy_path = "NON_IPM_SUBTREE/Freebusy%20Data/LocalFreebusy.EML";
-												 
+
 	g_object_ref (comp);
-												 
+
 	/* Look up the resource's mailbox */
 	gc = exchange_account_get_global_catalog (cbex->account);
 	if (!gc)
 		goto cleanup;
-												 
+
 	gcstatus = e2k_global_catalog_lookup (
 		gc, NULL, E2K_GLOBAL_CATALOG_LOOKUP_BY_EMAIL, resource_email,
 		E2K_GLOBAL_CATALOG_LOOKUP_MAILBOX, &entry);
@@ -1814,16 +1751,16 @@ book_resource (ECalBackendExchange *cbex,
 	switch (gcstatus) {
 		case E2K_GLOBAL_CATALOG_OK:
 			break;
-												 
+
 		case E2K_GLOBAL_CATALOG_NO_SUCH_USER:
 			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_NO_SUCH_USER;
 			goto cleanup;
-												 
+
 		default:
 			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
 			goto cleanup;
 	}
-												 
+
 	top_uri = exchange_account_get_foreign_uri (cbex->account,
 						    entry, NULL);
 	cal_uri = exchange_account_get_foreign_uri (cbex->account, entry,
@@ -1833,7 +1770,7 @@ book_resource (ECalBackendExchange *cbex,
 		retval = E_CAL_BACKEND_EXCHANGE_BOOKING_PERMISSION_DENIED;
 		goto cleanup;
 	}
-	
+
 	ctx = exchange_account_get_context (cbex->account);
 	status = e2k_context_propfind (ctx, NULL, cal_uri,
 					&prop_name, 1,
@@ -1849,7 +1786,7 @@ book_resource (ECalBackendExchange *cbex,
 		retval = E_CAL_BACKEND_EXCHANGE_BOOKING_PERMISSION_DENIED;
 		goto cleanup;
 	}
-												 
+
 	prop_name = PR_PROCESS_MEETING_REQUESTS;
 	iter = e2k_context_bpropfind_start (ctx, NULL, top_uri,
 						&localfreebusy_path, 1,
@@ -1871,23 +1808,23 @@ book_resource (ECalBackendExchange *cbex,
 
 	e_cal_component_get_uid (E_CAL_COMPONENT (comp), &uid);
 	href = g_strdup_printf ("%s/%s.EML", cal_uri, uid);
-	
+
 	e_cal_backend_exchange_cache_lock (cbex);
 	ecomp = get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbex), uid);
 	e_cal_backend_exchange_cache_unlock (cbex);
-												 
+
 	if (method == ICAL_METHOD_CANCEL) {
 		char *object = NULL;
-		
+
 		/* g_object_unref (comp); */
 		/* If there is nothing to cancel, we're good */
 		if (!ecomp) {
 			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_OK;
 			goto cleanup;
 		}
-		
+
 		/* Mark the cancellation properly in the resource's calendar */
-		
+
 		/* Mark the item as cancelled */
 		e_cal_component_get_summary (E_CAL_COMPONENT (comp), &old_text);
 		if (old_text.value)
@@ -1905,7 +1842,7 @@ book_resource (ECalBackendExchange *cbex,
 		} else {
 			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
 		}
-		
+
 		g_free (object);
 		goto cleanup;
 	} else {
@@ -1917,13 +1854,13 @@ book_resource (ECalBackendExchange *cbex,
 		tt = icaltime_as_timet_with_zone (*dt.value, izone);
 		e_cal_component_free_datetime (&dt);
 		startz = e2k_make_timestamp (tt);
-												 
+
 		e_cal_component_get_dtend (E_CAL_COMPONENT (comp), &dt);
 		izone = e_cal_backend_internal_get_timezone (E_CAL_BACKEND (cbex), dt.tzid);
 		tt = icaltime_as_timet_with_zone (*dt.value, izone);
 		e_cal_component_free_datetime (&dt);
 		endz = e2k_make_timestamp (tt);
-												 
+
 		prop_name = E2K_PR_CALENDAR_UID;
 		rn = e2k_restriction_andv (
 			e2k_restriction_prop_bool (
@@ -1940,7 +1877,7 @@ book_resource (ECalBackendExchange *cbex,
 			e2k_restriction_prop_string (
 				E2K_PR_CALENDAR_BUSY_STATUS, E2K_RELOP_NE, "FREE"),
 			NULL);
-												 
+
 		iter = e2k_context_search_start (ctx, NULL, cal_uri,
 						     &prop_name, 1, rn, NULL, FALSE);
 		g_free (startz);
@@ -1956,9 +1893,9 @@ book_resource (ECalBackendExchange *cbex,
 			status = e2k_result_iter_free (iter);
 
 			if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
-				if (status == E2K_HTTP_UNAUTHORIZED) 
+				if (status == E2K_HTTP_UNAUTHORIZED)
 					retval = E_CAL_BACKEND_EXCHANGE_BOOKING_PERMISSION_DENIED;
-				else 
+				else
 					retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
 			}
 			e2k_restriction_unref (rn);
@@ -1966,21 +1903,21 @@ book_resource (ECalBackendExchange *cbex,
 		}
 		e2k_restriction_unref (rn);
 	}
-												 
+
 	/* We're good. Book it. */
 	/* e_cal_component_set_href (comp, href); */
 
 	icalparameter_set_partstat (part_param, ICAL_PARTSTAT_ACCEPTED);
-	
+
 	e_cal_component_commit_sequence (comp);
 	calobj = (char *) e_cal_component_get_as_string (comp);
-												 
+
 	/* status = e_cal_component_update (comp, method, FALSE  ); */
 	if (ecomp) {
 		/* Use the PUT method to create the meeting item in the resource's calendar. */
 		status = modify_object_with_href (E_CAL_BACKEND_SYNC (cbex), cal, calobj, CALOBJ_MOD_THIS, &old_object, &new_object, href, NULL);
 		if (status == GNOME_Evolution_Calendar_Success) {
-			/* Need this to update the participation status of the resource 
+			/* Need this to update the participation status of the resource
 			   in the organizer's calendar. */
 			status = modify_object_with_href (E_CAL_BACKEND_SYNC (cbex), cal, calobj, CALOBJ_MOD_THIS, &old_object, &new_object, NULL, NULL);
 		} else {
@@ -2010,7 +1947,7 @@ book_resource (ECalBackendExchange *cbex,
 	if (top_uri)
 		g_free (top_uri);
 	g_free (calobj);
-					
+
 	return retval;
 }
 
@@ -2026,24 +1963,24 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 	icalcomponent *top_level, *icalcomp, *tzcomp;
 	icalproperty *prop;
 	icalproperty_method method;
-												 
-	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex), 
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex),
 				GNOME_Evolution_Calendar_InvalidObject);
 
-	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (cbex))) { 
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (cbex))) {
 		return GNOME_Evolution_Calendar_InvalidObject;
 	}
-												 
+
 	*users = NULL;
 	*modified_calobj = NULL;
 
 	top_level = icalparser_parse_string (calobj);
 	icalcomp = icalcomponent_new_clone (icalcomponent_get_inner (top_level));
-												 
+
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (E_CAL_COMPONENT (comp),
 					   icalcomp);
-												 
+
 	method = icalcomponent_get_method (top_level);
 	if (icalcomponent_isa (icalcomp) != ICAL_VEVENT_COMPONENT
 	    || (method != ICAL_METHOD_REQUEST && method != ICAL_METHOD_CANCEL)) {
@@ -2068,21 +2005,21 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 		icalvalue *value;
 		icalparameter *param;
 		const char *attendee;
-												 
+
 		param = icalproperty_get_first_parameter (prop, ICAL_CUTYPE_PARAMETER);
 		if (!param)
 			continue;
 		if (icalparameter_get_cutype (param) != ICAL_CUTYPE_RESOURCE)
 			continue;
-												 
+
 		value = icalproperty_get_value (prop);
 		if (!value)
 			continue;
-												 
+
 		attendee = icalvalue_get_string (value);
 		if (g_ascii_strncasecmp ("mailto:", attendee, 7))
 			continue;
-												 
+
 		/* See if it recurs */
 		if (icalcomponent_get_first_property (icalcomp, ICAL_RRULE_PROPERTY)
 		    || icalcomponent_get_first_property (icalcomp, ICAL_RDATE_PROPERTY)) {
@@ -2091,11 +2028,11 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 				_("Unable to schedule resource '%s' for recurring meetings.\n"
 				      "You must book each meeting separately."),
 				    attendee + 7);
-#endif												 
+#endif
 			retval = GNOME_Evolution_Calendar_ObjectIdAlreadyExists;
 			goto cleanup;
 		}
-												 
+
 		param = icalproperty_get_first_parameter (prop, ICAL_PARTSTAT_PARAMETER);
 		result = book_resource (cbex, cal, attendee + 7, comp, method, param);
 		switch (result) {
@@ -2108,10 +2045,10 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 			g_snprintf (error_msg, 256,
 				    _("The resource '%s' is busy during the selected time period."),
 				    attendee + 7);
-#endif							
+#endif
 			retval = GNOME_Evolution_Calendar_ObjectIdAlreadyExists;
 			goto cleanup;
-												 
+
 		case E_CAL_BACKEND_EXCHANGE_BOOKING_PERMISSION_DENIED:
 		case E_CAL_BACKEND_EXCHANGE_BOOKING_NO_SUCH_USER:
 			/* Do nothing, we fallback to iMip */
@@ -2122,15 +2059,15 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 			goto cleanup;
 		}
 	}
-												 
+
 	retval = GNOME_Evolution_Calendar_Success;
 	e_cal_component_commit_sequence (comp);
 	*modified_calobj = g_strdup (e_cal_component_get_as_string (comp));
-												 
+
  cleanup:
 	icalcomponent_free (top_level);
 	g_object_unref (comp);
-												 
+
 	return retval;
 }
 
@@ -2313,7 +2250,7 @@ get_free_busy (ECalBackendSync *backend, EDataCal *cal,
 	for (item = e2k_xml_find_in (recipients, recipients, "item");
 	     item;
 	     item = e2k_xml_find_in (item, recipients, "item")) {
-		icalcomponent *vfb;			
+		icalcomponent *vfb;
 		icalproperty *organizer;
 		xmlNode *node, *fbdata;
 		char *org_uri, *calobj;
